Package: CDSS
Type: Package
Title: Course-Dependent Skill Structures
Version: 0.3-1
Date: 2026-04-15
Depends: R (>= 4.3.0)
Imports: readODS (>= 2.0.0), openxlsx (>= 4.2.0)
Authors@R: c(person(given = "Cord", family = "Hockemeyer",
                    role = c("aut", "cre"),
                    email = "cord.hockemeyer@uni-graz.at"))
Maintainer: Cord Hockemeyer <cord.hockemeyer@uni-graz.at>
License: GPL-3
NeedsCompilation: no
Description: Deriving skill structures from skill assignment 
    data for courses (sets of learning objects).
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: litedown, DiagrammeRsvg, rsvg, png, kstMatrix
VignetteBuilder: litedown
Packaged: 2026-04-27 04:23:00 UTC; hockemey
Author: Cord Hockemeyer [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-27 05:20:02 UTC
Built: R 4.5.2; ; 2026-04-27 09:36:07 UTC; unix

Package: CDSS
Type: Package
Title: Course-Dependent Skill Structures
Version: 0.3-1
Date: 2026-04-15
Depends: R (>= 4.3.0)
Imports: readODS (>= 2.0.0), openxlsx (>= 4.2.0)
Authors@R: c(person(given = "Cord", family = "Hockemeyer",
                    role = c("aut", "cre"),
                    email = "cord.hockemeyer@uni-graz.at"))
Maintainer: Cord Hockemeyer <cord.hockemeyer@uni-graz.at>
License: GPL-3
NeedsCompilation: no
Description: Deriving skill structures from skill assignment 
    data for courses (sets of learning objects).
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: litedown, DiagrammeRsvg, rsvg, png, kstMatrix
VignetteBuilder: litedown
Packaged: 2026-04-27 04:23:00 UTC; hockemey
Author: Cord Hockemeyer [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-27 05:20:02 UTC
Built: R 4.6.0; ; 2026-05-01 21:07:32 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cdss_binary_matrix_product)
export(cdss_circular_requirements)
export(cdss_close_ar)
export(cdss_csma2sf)
export(cdss_lo_csma2sf)
export(cdss_lo_sa2af)
export(cdss_lo_sa2ar)
export(cdss_missing_los)
export(cdss_nonteaching_los)
export(cdss_read_skill_assignment_csv)
export(cdss_read_skill_assignment_ods)
export(cdss_read_skill_assignment_xlsx)
export(cdss_reduce_sf)
export(cdss_sa2ar_skill)
export(cdss_sa2sma)
export(cdss_sa_compliance)
export(cdss_sa_describes_sr)
export(cdss_sma2csma)
export(cdss_tables2sa)
export(cdss_wf_read_skill_assignment)
importFrom(openxlsx,read.xlsx)
importFrom(readODS,read_ods)
importFrom(tools,file_ext)
importFrom(utils,read.csv)

library(CDSS)
library(kstMatrix)
# library(knitr)

fpath <- system.file("extdata", "SkillAssignment.xlsx", package="CDSS")
sflist <- cdss_wf_read_skill_assignment(fpath)
sflist
b <- kstMatrix::kmbasis(sflist$sfs)
b

pl <- plot(b)
# rsvg::rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), paste0(tempdir(), file="/plot3.png", width=2000))
bm <- rsvg::rsvg(charToRaw(DiagrammeRsvg::export_svg(pl)), width=1500)
png::writePNG(bm, paste0(tempdir(), file="/plot3.png"))

fpath <- system.file("extdata", "SkillAssignment-T.csv", package="CDSS")
read.csv(fpath)
fpath <- system.file("extdata", "SkillAssignment-R.csv", package="CDSS")
read.csv(fpath)

fpath <- system.file("extdata", "SkillAssignment_SR.xlsx", package="CDSS")
sa <- cdss_read_skill_assignment_xlsx(fpath)
sa
cdss_sa_describes_sr(sa)

ar <- cdss_sa2ar_skill(sa)
ar
sr <- cdss_close_ar(ar)
sr

pl <- plot(sr)
# rsvg::rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), paste0(tempdir(), file="/plot4.png", width=1000))
bm <- rsvg::rsvg(charToRaw(DiagrammeRsvg::export_svg(pl)), width=1000)
png::writePNG(bm, paste0(tempdir(), file="/plot4.png"))

#' Read an assignment of taught and required skills for a set of learning objects from CSV-files.
#'
#' \code{cdss_read_skill_assignment} expects two CSV-files with two columns each. The first
#' column contains the IDs of learning objects and the second row the IDs of single skills
#' required or taught, respectively, by this learning object.
#' It returns a list of two binary matrices, "taught" and "required". Each matrix has one
#' row per learning object and one column per skill. The cells contain a "1" if the skill
#' is taught or required, respectively, by the learning object and a "0" otherwise,
#'
#' @param taught CSV-file with assignments of taught competencies to learning objects
#' @param required CSV-file with assignments of required competencies to learning objects
#' @param header Boolean specifying whether the CSV-files contain a header line (default = TRUE)
#' @param sep Column separator (default ",")
#' @param dec Decimal point character (default ".")
#' @param warnonly Are non-compliant SAs allowed? (default = FALSE)
#' @param verbose Verbosity of compliance test (default = TRUE)
#'
#' @return List of two binary matrices, "taught" and "required".
#' 
#' @importFrom utils read.csv
#'
#' @family functions reading skill assignments
#'
#' @export
cdss_read_skill_assignment_csv <- function(
    taught,
    required,
    header = TRUE,
    sep = ",",
    dec = ".",
    warnonly = FALSE,
    verbose = TRUE
    )
{
  t <- read.csv(taught, header=header, sep=sep, dec=dec, fill=FALSE)
  r <- read.csv(required, header=header, sep=sep, dec=dec, fill=FALSE)
  sa <- cdss_tables2sa(t, r)
  check <- cdss_sa_compliance(sa, verbose)
  if (!check) {
    if (warnonly) {
      stop("The assignment tables are not skill assignment compliant!")
    } else {
      warning("The assignment tables are not skill assignment compliant!")
      
    }
  }
  sa
}

#' Close an attribution relation to get a surmise relation.
#'
#' \code{cdss_close_ar} expects a quadratic binary matrix and closes it under
#' reflexivity and transitivity.
#'
#' @param ar Attribution relation matrix
#'
#' @return surmise relation or NULL
#'
#' @family Utility functions
#'
#' @export
cdss_close_ar <- function(ar) {
  if (!(inherits(ar, "matrix"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("ar"),
                 dQuote("matrix")))
  }
  if ((max(ar) > 1) || (min(ar) < 0)) {
    stop(sprintf("%s is not a binary matrix!", dQuote("ar")))
  }
  size <- dim(ar)[1]
  if (size != dim(ar)[2]) {
    stop(sprintf("%s is not a uqadratic matrix!", dQuote("ar")))
  }
  
  # Close ar under reflexivity
  d <- diag(1,size,size)
  ar <- 1 * (ar | d)

  # Close ar under transitivity
  sr <- cdss_binary_matrix_product(ar, ar)
  sr.old <- ar
  while(any(sr != sr.old)) {
    sr.old <- sr
    sr <- cdss_binary_matrix_product(sr, sr)
  }
  colnames(sr) <- colnames(ar)
  rownames(sr) <- rownames(ar)
  class(sr) <- unique(c("kmsurmiserelation", class(sr)))
  sr
}

#' Build matrices of taught and required, respectively, skills for learning objects from
#' respective tables.
#'
#' \code{cdss_tables2sa} expects two data frames with two columns each. The first
#' column contains the IDs of learning objects and the second row the IDs of single skills
#' required or taught, respectively, by this learning object.
#' It returns a list of two binary matrices, "taught" and "required". Each matrix has one
#' row per learning object and one column per skill. The cells contain a "1" if the skill
#' is taught or required, respectively, by the learning object and a "0" otherwise.
#'
#' @param taught Data table containing the assignment of taught skills to learning objects
#' @param required Data table containing the assignment of required skills to learning objects
#'
#' @return List of two binary matrices, "taught" and "required".
#'
#' @family functions building skill (multi) assignment matrices
#'
#' @export
cdss_tables2sa <- function(taught, required) {
  # Doing some awful hack here: blowing up the skill and LO names to 128 chars width
  # and at the end trimming them back - but I did not get it to work otherwise
  skills <- trimws(format(unique(c(taught[,2],required[,2])), width=128, justify="right"), "both")
  los <- trimws(format(unique(c(taught[,1],required[,1])), width=128, justify="right"), "both")
  req <- matrix(data = rep(0, length(skills)*length(los)), nrow = length(los))
  tgt <- matrix(data = rep(0, length(skills)*length(los)), nrow = length(los))
  colnames(req) <- skills
  rownames(req) <- los
  colnames(tgt) <- skills
  rownames(tgt) <- los
  apply(required, MARGIN =1, FUN=function(x){req[trimws(format(x[1], width=35,justify="right"), "both"),
                                                 trimws(format(x[2], width=35,justify="right"), "both")] <<- 1})
  apply(taught, MARGIN =1, FUN=function(x){tgt[trimws(format(x[1], width=35,justify="right"), "both"),
                                               trimws(format(x[2], width=35,justify="right"), "both")] <<- 1})
  l <- list(taught=tgt, required=req)
  class(l) <- unique(c("cdss_sa", class(l)))
  l
}

recurseList <- function(skills, hmat, tgt) {
  if ((is.null(skills)) || (length(skills) == 0)) return(hmat)
  if (length(skills) > 1) {
    currSkill <- skills[1]
    restskills <- skills[2:length(skills)]
  } else {
    currSkill <- skills
    restskills = NULL
  }
  LOs <- rownames(tgt)[which(tgt[,currSkill]==1)]
  if (length(LOs) == 1) LOs <- list(LOs)
  result = matrix(, ncol = dim(hmat)[2], nrow = 0)
  colnames(result) <- colnames(hmat)
  sapply(LOs, function(x) {
    bmat <- hmat
    bmat[,x] <- 1
    bmat <- recurseList(restskills, bmat, tgt)
    result <<- rbind(result, bmat)
  })
  return(result)
}

#' Determine Attribution function for LOs from a skill assignment
#' 
#' \code{cdss_lo_sa2af} expects a skill assignment and derives an attribution
#' function on the set of learning objects.
#' 
#' @param sa Skill assignment object
#'
#' @return Object of class \code{cdss_af} (attribution function).
#'
#' @family functions building skill (multi) assignment matrices
#'
#' @export
cdss_lo_sa2af <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  req <- sa$required
  tgt <- sa$taught
  countLO <- dim(req)[1]
  af <- data.frame(t(c("xyz", 1:countLO))) # We will delete this row later
  colnames(af) <- c("LO", rownames(req))
  sapply(rownames(req), function(x) {
    skills <- names(which(req[x,]==1))
    hmat <- matrix(rep(0, countLO), nrow=1)
    colnames(hmat) <- rownames(tgt)
    hmat[1,x] <- 1
    hmat <- recurseList(skills, hmat, tgt)
    len <- dim(hmat)[1]
    los <- data.frame(matrix(rep(x, len), ncol=1))
    colnames(los) <- c("LO")
    af <<- rbind(af, cbind(los, hmat))
  })
  colnames(af) <- c("LO", rownames(req))
  class(af) <- unique(c("cdss_af", class(af)))
  return(af[-1,])
}

#' Complete a skill multi-assignment
#'
#' \code{cdss_sma2csma} expects a skill multi-assignment object and returns
#' the corresponding complete skill multi-assignment. 
#' If this would involve cycles, the function stops by default - except if 
#' \code{allowcycles} is set to \code{TRUE}. In that case, the result may
#' be ill-defined!
#'
#' @param sma Skill multi-assignment to be completed
#' @param allowcycles Whether prerequisite cycles should be allowed (default = FALSE)
#'
#' @return Object of class \code{cdss_csma}.
#'
#' @export
cdss_sma2csma <- function(sma, allowcycles = FALSE) {
  if (!(inherits(sma, "cdss_sma"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sma"),
                 dQuote("cdss_sma")))
  }

  skills <- (dim(sma)[2] - 1) / 2
  reqcols <- (skills+2):((2*skills)+1)
  cn <- colnames(sma)
  
  # We do only/max. one addition per outer loop
  chgd <- TRUE
  foundcycles <- FALSE
  clo <- NULL  # chgd LO
  while (chgd && (!foundcycles)) {
    # loop for a transitive extension
    chgd <- FALSE
    clauses <- dim(sma)[1]
    # for each row in the data frame
    lapply((1:clauses), function(cl) {
      if ((!chgd) && (!foundcycles)) {
        if (allowcycles) {
          cl_h <- 1 * (as.numeric(sma[cl,2:(skills+1)]) | as.numeric(sma[cl,(skills+2):((2*skills)+1)]))
        } else {
          cl_h <- as.numeric(sma[cl,(skills+2):((2*skills)+1)])
        }
        # for each required skill
        lapply(which(sma[cl,reqcols]==1), function(s) {
          if ((!chgd) && (!foundcycles)) {
            # possible extensions, i.e. rows for LOs teaching skill s
            poss_rows <- which(sma[,s+1] == 1)
            poss <- 1 * (sma[poss_rows,2:(skills+1)] | sma[poss_rows,(skills+2):((2*skills)+1)])
            # do we have any possible extension already included in cl_h?
            if (any(apply(poss, 1, function(p) {all(1*(p & cl_h) == p)}))) {
            } else {
              chgd <<- TRUE
              tgt <- 1 * (sma[cl,2:(skills+1)])
              req <- 1 * (sma[cl,(skills+2):((2*skills)+1)])
              clo <<- unlist(sma[cl,1])
              sma <<- sma[-cl,]
              apply(poss, 1, function(p) {
                if ((allowcycles) || (all((tgt & p) == 0))) {
                  v <- c(clo, tgt, 1*(req|p))
                  names(v) <- colnames(sma)
                  sma <<- data.frame(rbind(sma, t(v)))
                  sma[,2:(2*skills+1)] <<- sapply(sma[,2:(2*skills+1)], as.numeric)
                  rownames(sma) <<- 1:dim(sma)[1]
                } else
                  foundcycles <<- TRUE
              })
              if ((!allowcycles) && foundcycles) {
                stop(sprintf("Cycle(s) around LO %s! Result is undefined", clo))
              }
            }
          }
        })
      }
    })
    if ((!allowcycles) && foundcycles)
      return()
    # Remove comparable rows
    colnames(sma) <- cn
    rownames(sma) <- 1:(dim(sma)[1])
    if (chgd) {
      ch2 <- TRUE
      while (ch2) {
        rownames(sma) <- 1:(dim(sma)[1])
        ch2 <- FALSE
        sel <- which(sma[,1] == clo)
        lapply(sel, function(x) {
          if (!ch2) {
            lapply(sel, function(y) {
              if (!ch2) {
                if (x != y) {
                  if (all((sma[x,reqcols] & sma[y,reqcols]) == sma[x,reqcols])) {
                    sma <<- sma[-y,]
                    ch2 <<- TRUE
                  } else {
                  }
                }
              }
            })
          }
        })
        colnames(sma) <- cn
        rownames(sma) <- 1:(dim(sma)[1])
      }
    }
  }
  # Final touches to the result
  colnames(sma) <- cn
  rownames(sma) <- 1:(dim(sma)[1])
  class(sma) <- unique(c("cdss_csma", "cdss_sma", class(sma)))
  sma
} 

#' Convert skill assignment matrices to skill multi-assignment
#'
#' \code{cdss_sa2sma} expects a list of two matrices (\code{taught} and \code{required}) of a skill
#' assignment. It returns a skill multi-assignment object.
#'
#' @param sa Skill assignment object
#'
#' @return Object of class \code{cdss_sma}.
#'
#' @family functions building skill (multi) assignment matrices
#'
#' @export
cdss_sa2sma <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  df <- cbind(data.frame(rownames(sa$taught)),
              data.frame(sa$taught),
              data.frame(sa$required))
  colnames(df)[1] <- "LO"
  rownames(df) <- NULL
  class(df) <- unique(c("cdss_sma", class(df)))
  df
}
  

#' CDSS: Course dependent skill structures
#' 
#' The \code{CDSS} package provides functions for a complete workflow from
#' skill assignment tables to surmise mappings on the sets of skills and
#' learning objects, respectively.
#'
#' @section Suggested workflow for the general case:
#' 1. Read the skill assignment using one of the \code{read_skill_assignments_xxx()} functions.
#' 1. Check the compliance to the definition for skill assignments using [cdss_sa_compliance()].
#' 1. Convert the skill assignment into a skill multi-assignment using [cdss_sa2sma()].
#' 1. Close the skill multi-assignment under completion using [cdss_sma2csma()].
#' 1. Compute the surmise function on skills using [cdss_csma2sf()].
#' 1. Continue with functions from the \code{kstMatrix} package, e.g., to obtain a basis and further on a skill space.
#' 
#' @section Suggested workflow for the special case of one LO per skill:
#' 1. Read the skill assignment using one of the \code{read_skill_assignments_xxx()} functions.
#' 1. Check whether the skill assignment allows for the derivation of a surmise relation using [cdss_sa_describes_sr()].
#' 1. If yes, derive an attribution relation from the skill assignment using [cdss_sa2ar_skill()].
#' 1. Close the attribution relation to a surmise relation using [cdss_close_ar()].
#' 1. Continue with functions from the \code{kstMatrix} package, e.g., to obtain a basis and further on a skill space.
#' 
#' @section Data files:
#' The installation of this package includes several data files as examples in the
#' \code{extdata} sub directory (see the Examples below for how to access the files there). 
#' There are four data sets, \code{KST}, \code{KST-Intro}, \code{SkillAssignment}, and
#' \code{ErroneousSkillAssignment}. The \code{SkillAssignment} data set is available in 
#' three formats, ODS, XLSX, and CSV (in CSV format, there are two files each, 
#' \code{SkillAssignment-R.csv} and \code{SkillAssignment-T.csv}, for required and taught skills, respectively).
#' The other three data sets are available in ODS format only.
#' 
#' \code{SkillAssignment} and \code{ErroneousSkillAssignment} are small example data sets
#' where the latter fails for \code{cdss_sa_compliance()}. \code{KST} contains a skill
#' assignment for the course on knowledge space theory under <https://moodle.qhelp.eu/>.
#' \code{KST-Intro} contains the reduction of \code{KST} to the first chapter of
#' that course.
#' 
#' @examples
#' library(readODS)
#' fpath <- system.file("extdata", "SkillAssignment.ods", package="CDSS")
#' sa <- cdss_read_skill_assignment_ods(fpath)
#' sa
#' sma <- cdss_sa2sma(sa)
#' sma
#' csma <- cdss_sma2csma(sma)
#' csma
#' sf <- cdss_csma2sf(csma)
#' sf
#'
#' @section References:
#' Hockemeyer, C. (2022). Building Course-Dependent Skill Structures - Applying
#' Competence based Knowledge Space Theory to Itself. Manuscript in preparation.
#' 
#' @section Acknowledgements:
#' The creation of this R package was financially supported by the Erasmus+ Programme
#' of the European Commission through the QHELP project (<https://qhelp.eu/>).
#'
#' @aliases CDSS-package
#' @md
#' @name CDSS
NULL

#' Compute a binary matrix product
#'
#' \code{cdss_binary_matrix_product} expects two binary matrices and computes there Boolean product.
#'
#' @param m Binary matrix
#' @param n Binary matrix
#'
#' @return Boolean matrix product of m and n
#' 
#' @family Utility functions
#'
#' @export
cdss_binary_matrix_product <- function(m,n) {
  if (dim(m)[1] != dim(n)[2]) {
    stop(sprintf("%s and %s do not fit in size!", dQuote("m"), dQuote("n")))
  }
  res <- matrix(rep(0,dim(m)[2]*dim(n)[1]), nrow=dim(m)[2])
  lapply((1:dim(m)[2]), function(x) {
    lapply((1:dim(n)[1]), function(y) {
      res[x,y] <<- max(m[x,] & n[,y])
    })
  })
  res
}

even <- function(i) {
  if (floor(i/2)*2 == i) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Derive a surmise function between learning objects from a complete skill multi-assignment
#' 
#' \code{cdss_lo_csma2sf} expects a complete skill multi-assignment and derives a surmise
#' function on the set of learning objects.
#' 
#' @param csma Complete skill multi-assignment object
#'
#' @return Object of class \code{cdss_sf} (attribution function).
#'
#' @family functions building skill (multi) assignment matrices
#'
#' @export
cdss_lo_csma2sf <- function(csma) {
  if (!(inherits(csma, "cdss_csma"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("csma"),
                 dQuote("cdss_csma")))
  }
  colcnt <- dim(csma)[2]
  if (even(colcnt)) {
    stop(sprintf("Internal error: A CSMA cannot have an even number of columns!"))
  }
  skillcnt <- colcnt / 2
  tgt <- as.matrix(csma[,2:((skillcnt+1))])
  req <- as.matrix(csma[,((skillcnt+1):dim(csma)[2])])
  rownames(tgt) <- csma[,1]
  rownames(req) <- csma[,1]
  sa <- list(taught = tgt, required=req)
  class(sa) <- unique(c("cdss_sa", class(sa)))
  rtaf <- cdss_lo_sa2af(sa)
  chgd <- TRUE
  locnt <- dim(rtaf)[2]-1
  LOs <- colnames(rtaf)[-1]
  rtaf[,2:(locnt+1)] <- sapply(rtaf[,2:(locnt+1)], as.numeric)
  while (chgd) {
    chgd <- FALSE
    sapply(LOs, function(l) {
      if (!chgd) {
        sel <- which(rtaf[,1]==l) 
        lapply(sel, function(x) {
          if (!chgd) {
            lapply(sel, function(y) {
              if ((!chgd) & (x != y)) {
                if (all((rtaf[x,2:(locnt+1)] & rtaf[y,2:(locnt+1)]) == rtaf[x,2:(locnt+1)])) {
                  rtaf <<- rtaf[-y,]
                  chgd <<- TRUE
                }
              }
            })
          }
        })
      } 
    })
  }
  class(rtaf) <- c("kmsurmisefunction", class(rtaf))
  rtaf
}

#' Check whether a surmise relation can be derived from a given skill assignment.
#'
#' \code{cdss_sa_describes_sr} expects a list of two matrices (\code{taught} and \code{required}) of a skill
#' assignment. It returns TRUE if the skill assignment describes a surmise relation (i.e. there is
#' only one teaching LO per skill) and FALSE.
#'
#' @param sa Skill assignment object
#' @param verbose Flag, default is FALSE
#'
#' @return Logical value
#'
#' @family functions deriving skill structures from skill assignments
#'
#' @export
cdss_sa_describes_sr <- function(sa, verbose = FALSE) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  rv <- max(colSums(sa$taught)) == 1
  if ((!rv) & verbose) {
    c <- colnames(sa$taught)[which(colSums(sa$taught) > 1)]
    msg <- paste ("The following skills have more than one teaching LO:", c)
    message(msg)
  }
  rv
}

#' Check whether a skill assignment is compliant to the CDCS conditions.
#'
#' \code{cdss_sa_compliance} expects a skill assignment and checks whether
#' it is compliant to the conditions for CDCS.
#'
#' @param sa Skill assignment
#' @param warnings Toggles whether warnings should be printed
#'
#' @return Boolean
#' 
#' @family Functions testing validity of skill assignments
#'
#' @export
cdss_sa_compliance <- function(sa, warnings = FALSE) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  taught <- sa$taught
  required <- sa$required
  # Checks if matrices fit together
  if (!(all(dim(taught) == dim(required)))) {
    stop(sprintf("%s and %s must have equal size!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(colnames(required) == colnames(taught)))) {
    stop(sprintf("%s and %s must have equal colnames!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(rownames(required) == rownames(taught)))) {
    stop(sprintf("%s and %s must have equal rownames!", dQuote("taught", dQuote("required"))))
  }

  result <- TRUE

  if (any(colSums(taught) == 0)) {
    if (warnings) warning("Not all skills are taught by learning objects: ",
            names(which(colSums(taught) == 0)))
    result <- FALSE
  }

  if (any(required & taught)) {
    is <- rownames(which(required & taught, arr.ind=TRUE))
    if (warnings) warning("Some skills are taught and required for the same learning object: ", is)
    result <- FALSE
  }
  
  if (any(rowSums(taught) == 0)) {
    if (warnings) warning("Not all learning objects teach something: ",
                          names(which(rowSums(taught) == 0)))
    result <- FALSE
  }

  result
}

#' Create an attribution relation on skills from a skill assignment.
#'
#' \code{cdss_sa2ar_skill} expects a skill assignment and derives an attribution relation
#'  on skills if the skill assignment fulfills the necessary conditions, i.e. if there 
#'  is only one teaching LO per skill.
#'
#' @param sa Skill assignment object
#'
#' @return attribution relation or NULL
#'
#' @family functions deriving skill structures from skill assignments
#'
#' @export
cdss_sa2ar_skill <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  if (!cdss_sa_describes_sr(sa)) {
    stop(sprintf("%s does not describe a surmise relation!", dQuote("sa")))
  }
  
  # Determine attribution relation
  skills <- dim(sa$taught)[2]
  ar <- diag(1, skills, skills)
  lapply(rownames(sa$taught), function(x) {
    sapply((1:skills), function(y) {
      if (sa$taught[x,y]) {
        sapply((1:skills), function(z) {
          if (sa$taught[x,z]) {ar[y,z] <<- 1}
        })
        sapply((1:skills), function(z) {
          if (sa$required[x,z]) {ar[y,z] <<- 1}
        })
      }
    })
  })
  colnames(ar) <- colnames(sa$taught)
  rownames(ar) <- colnames(sa$taught)
  class(ar) <- unique(c("attribution_relation", class(ar)))
  t(ar)
}

library(openxlsx)
#' Read an assignment of taught and required skills for a set of learning objects from an XLSX-file.
#'
#' \code{cdss_read_skill_assignment_xlsx} expects an XLSX-file with two sheets assigning taught and
#' required, respectively, skills to learning objects with two columns each. The first
#' column contains the IDs of learning objects and the second row the IDs of single skills
#' required or taught, respectively, by this learning object.
#' It returns a list of two binary matrices, "taught" and "required". Each matrix has one
#' row per learning object and one column per skill. The cells contain a "1" if the skill
#' is taught or required, respectively, by the learning object and a "0" otherwise,
#'
#' @param filename Name of the XLSX-file
#' @param taughtname Name of the sheet with required assignment (default = "Taught")
#' @param requiredname Name of the sheet with required assignment (default = "Required")
#' @param warnonly Are non-compliant SAs allowed? (default = FALSE)
#' @param verbose Verbosity of compliance test (default = TRUE)
#'
#' @return List of two binary matrices, "taught" and "required".
#' 
#' @importFrom openxlsx read.xlsx
#'
#' @family functions reading skill assignments
#'
#' @export
cdss_read_skill_assignment_xlsx <- function(
    filename,
    taughtname = "Taught",
    requiredname = "Required",
    warnonly = FALSE,
    verbose = TRUE
)
{
  t <- read.xlsx(xlsxFile = filename, sheet = taughtname)
  r <- read.xlsx(xlsxFile = filename, sheet = requiredname)
  sa <- cdss_tables2sa(t, r)
  check <- cdss_sa_compliance(sa, verbose)
  if (!check) {
    if (warnonly) {
      stop("The assignment tables are not skill assignment compliant!")
    } else {
      warning("The assignment tables are not skill assignment compliant!")
    }
  }
  sa
}

#' Reduce a surmise function with respect to item equivalence
#'
#' \code{cdss_reduce_sf} takes a surmise function and returns its
#' reduction to non-equivalent items.
#'
#' @param sf Surmise function
#' @return Surmise function reduced by equivalences
#'
#' @family Utility functions
#'
#' @export
cdss_reduce_sf <- function(sf) {
  if (!inherits(sf, "cdss_sf")) {
    stop(sprintf("%s must be of class %s.", dQuote("sf"), dQuote("cdss_sf")))
  }

  sf <- t(sf)
  sf <- unique(sf)
  sf <- data.frame(t(unique(sf, MARGIN = 2)))
  rem <- colnames(sf)[-1]
  rownames(sf) <- sf[,1]
  sf <- sf[rem,]
  skills <- dim(sf)[2]-1
  sf <- data.frame(apply(sf[, 2:(skills+1)], 2, as.numeric))
  sf <- cbind(data.frame(rem), sf)
  colnames(sf)[1] <- "Skill"
  sf
}

#' Vector of learning objects requiring and teaching the same skill
#'
#' \code{cdss_circular_requirements} expects skill assignment and returns
#' a vector of learning objects which require a skill that they teach.
#'
#' @param sa Skill assignment
#'
#' @return Vector of learning objects
#' 
#' @family Functions testing validity of skill assignments
#'
#' @export
cdss_circular_requirements <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  required <- sa$required
  taught <- sa$taught
  
  # Checks if matrices fit together
  if (!(all(dim(taught) == dim(required)))) {
    stop(sprintf("%s and %s must have equal size!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(colnames(required) == colnames(taught)))) {
    stop(sprintf("%s and %s must have equal colnames!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(rownames(required) == rownames(taught)))) {
    stop(sprintf("%s and %s must have equal rownames!", dQuote("taught", dQuote("required"))))
  }

  rownames(which(required & taught, arr.ind=TRUE))
}

#' Vector of learning objects not teaching any skills.
#'
#' \code{cdss_nonteaching_los} expects a skill assignment and returns a vector
#' of learning objects which do not teach any skill.
#'
#' @param sa SKill assignment
#'
#' @return Vector of learning objects
#' 
#' @family Functions testing validity of skill assignments
#'
#' @export
cdss_nonteaching_los <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  taught <- sa$taught
  required <- sa$required
  
  # Checks if matrices fit together
  if (!(all(dim(taught) == dim(required)))) {
    stop(sprintf("%s and %s must have equal size!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(colnames(required) == colnames(taught)))) {
    stop(sprintf("%s and %s must have equal colnames!", dQuote("taught", dQuote("required"))))
  }
  if (!(all(rownames(required) == rownames(taught)))) {
    stop(sprintf("%s and %s must have equal rownames!", dQuote("taught", dQuote("required"))))
  }

  names(which(rowSums(taught) == 0))

}

library(readODS)
library(openxlsx)
library(tools)
#' Read an assignment of taught and required skills for a set of learning objects from file and
#' do the whole workflow up to a surmise function on skills
#'
#' \code{cdss_wf_read_skill_assignment} expects an ODS or XLSX file with two sheets assigning 
#' taught and required, respectively, skills to learning objects with two columns each. 
#' Alternatively, two CSV files can be specified. In the sheets/CSV files, the first
#' column contains the IDs of learning objects and the second row the IDs of single skills
#' required or taught, respectively, by this learning object.
#' It returns a list of two binary matrices, "taught" and "required". Each matrix has one
#' row per learning object and one column per skill. The cells contain a "1" if the skill
#' is taught or required, respectively, by the learning object and a "0" otherwise,
#'
#' @param filename Name of the file (in case of CSV files the one with TAUGHT assignments)
#' @param filename2 Name of the CSV file with REQUIRED assignments (if applicable)
#' @param filetype Type of the file, allowed values are "auto", "ODS", "XLSX", and "CSV"
#' @param taughtname Name of the sheet with required assignment (default = "Taught")
#' @param requiredname Name of the sheet with required assignment (default = "Required")
#' @param header Boolean specifying whether the CSV-files contain a header line (default = TRUE)
#' @param sep Column separator for CSV files (default ",")
#' @param dec Decimal point character for CSV files (default ".")
#' @param warnonly Are non-compliant SAs allowed? (default = FALSE)
#' @param verbose Verbosity of compliance test (default = TRUE)
#'
#' @return List of four elements: sfs (surmise function between skills),
#'                                sfl (surmise function between learning objects)
#'                                srs (surmise relation between skills, if available; NULL otherwise)
#'                                srl (surmise relation between learning objectrs, if available; NULL otherwise)
#' 
#' @importFrom readODS read_ods
#' @importFrom tools file_ext
#' @importFrom utils read.csv
#' @importFrom openxlsx read.xlsx
#' 
#' @family functions reading skill assignments
#'
#' @export
cdss_wf_read_skill_assignment <- function(
    filename,
    filename2 = NULL,
    filetype = "auto",
    taughtname = "Taught",
    requiredname = "Required",
    header = TRUE,
    sep = ",",
    dec = ".",
    warnonly = FALSE,
    verbose = TRUE
)
{
 if (filetype == "auto") {
   if (!(is.null(filename2))) {
     filetype = "CSV"
     sa <- cdss_read_skill_assignment_csv(filename, filename2, header, sep, dec, warnonly, verbose)
   } else {
     ext <- tolower(file_ext(filename))
     if (ext == "ods") {
       filetype <- "ODS"
       sa <- cdss_read_skill_assignment_ods(filename, taughtname, requiredname, warnonly, verbose)
     } else if (ext == "xlsx") {
       filetye <- "XLSX"
       sa <- cdss_read_skill_assignment_xlsx(filename, taughtname, requiredname, warnonly, verbose)
     } else if (ext == "csv")
       stop("For CSV filetype, two filenames must be specified!")
     else
       stop("Unknown filetype specified!")
   }
 }
  sma <- cdss_sa2sma(sa)
  csma <- cdss_sma2csma(sma)
  sfs <- cdss_csma2sf(csma)
  sfl <- cdss_lo_csma2sf(csma)
  if (cdss_sa_describes_sr(sa)) {
    srs <- cdss_sa2ar_skill(sa)
    srl <- cdss_lo_sa2ar(sa)
  } else {
    srs <- NULL
    srl <- NULL
  }
  list (sfs=sfs, sfl=sfl, srs=srs, srl=srl)
}
  

#' Create an attribution relation on learning objects from a skill assignment.
#'
#' \code{cdss_lo_sa2ar} expects a skill assignment and derives an attribution relation
#'  on learning objects if the skill assignment fulfills the necessary conditions, i.e. if  
#'  there is only one teaching LO per skill.
#'
#' @param sa Skill assignment object
#'
#' @return attribution relation or NULL
#'
#' @family functions deriving skill structures from skill assignments
#'
#' @export
cdss_lo_sa2ar <- function(sa) {
  if (!(inherits(sa, "cdss_sa"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("sa"),
                 dQuote("cdss_sa")))
  }
  if (!cdss_sa_describes_sr(sa)) {
    stop(sprintf("%s does not describe a surmise relation!", dQuote("sa")))
  }

  los <- unique(rownames(sa$taught))
  ar <- diag(length(los))
  colnames(ar) <- rownames(sa$taught)
  rownames(ar) <- rownames(sa$taught)
  sapply(rownames(sa$taught), function(lo) {
    sapply(colnames(sa$taught), function(skill) {
      if (sa$required[lo,skill] == 1) {
        lo2 <- names(which(sa$taught[,skill] == 1))
        ar[lo2,lo] <<- 1
      }
    })
  })
  ar
} 

#' Derive a surmise function from a complete skill multi-assignment
#'
#' \code{cdss_csma2sf} expects a complete skill multi-assignment object 
#' and returns the corresponding surmise function on the set of skills.
#'
#' @param csma Skill multi-assignment to be completed
#'
#' @return Object of class \code{cdss_csma}.
#'
#' @export
cdss_csma2sf <- function(csma) {
  if (!(inherits(csma, "cdss_csma"))) {
    stop(sprintf("%s must be of class %s.",
                 dQuote("csma"),
                 dQuote("cdss_csma")))
  }
  # First, we start with an attribution function.
  # This AF will be closed under reflexivity and transitivity because
  #   we start this from a CSMA.
  skills <- (dim(csma)[2] - 1) / 2
  tgtcols <- 2:(skills+1)
  reqcols <- (skills+2):(2*skills+1)
  
#  sl <- colnames(csma)[tgtcols]
  sf <- data.frame(t(c("xyz", 1:skills))) # We will delete this row later
  colnames(sf) <- 1:(skills+1)
  lapply(1:skills, function(s) {
    currskill <- colnames(csma)[s+1]
    clausenums <- which(csma[,s+1]==1)
    m1 <- matrix(rep(currskill, length(clausenums)),ncol = 1)
    m2 <- 1 * (csma[clausenums,tgtcols] | csma[clausenums,reqcols])
    hf <- data.frame(cbind(m1, m2))
    colnames(hf) <- 1:(skills+1)
    sf <<- rbind(sf, hf)
  })
  sf <- sf[-1,]
  # Now we have to reach incomparability
  chgd <- TRUE
  sf[,2:(skills+1)] <- sapply(sf[,2:(skills+1)], as.numeric)
  while (chgd) {
    chgd <- FALSE
    rownames(sf) <- 1:dim(sf)[1]
    lapply(unique(sf[,1]), function(s) {
      if (!chgd) {
        sel <- which(sf[,1] == s)
        lapply(sel, function(x) {
          if (!chgd) {
            lapply(sel, function(y) {
              if (!chgd) {
                if (x != y) {
                  if (all((sf[x,2:(skills+1)] & sf[y,2:(skills+1)]) == sf[x,2:(skills+1)])) {
                    sf <<- sf[-y,]
                    chgd <<- TRUE
                  }
                }
              }
            })
          }
        })
      }
    })
  }
  colnames(sf) <- c("Skill", colnames(csma)[tgtcols])
  class(sf) <- unique(c("kmsurmisefunction", class(sf)))
  sf
}

---
title: "CDSS"
subtitle: "Course--Dependent Skill Structures"
author: "Cord Hockemeyer"
date: "`{r} format(Sys.time(), '%B %d, %Y')`"
output: 
  html:
    options:
      number_sections: true
      fig_caption: true
      highlight: true
      toc: 
        depth: 3
vignette: >
  %\VignetteIndexEntry{CDSS}
  %\VignetteEngine{litedown::vignette}
  %\VignetteEncoding{UTF-8}
abstract: "The `CDSS` package provides A KISS (keep it simple, stupid) approach to derive a skill structure for an existing set of learning objects. Basis for the skill structure is the assignment of taught and required skills, respectively, to each learning object."
keywords: "knowledge space theory, skill structure, learning objects"
---

```{r setup, echo=FALSE}
library(CDSS)
library(kstMatrix)
# library(knitr)
```
# Introduction
The `CDSS` package (course dependent skill structures) provides a way to quickly develop a rough competence structure 
for an existing course. The basic idea is to identify, for each lesson of the course, the skills taught by that lesson 
and required for understanding that lessons, respectively. The interesting point is that only the _direct_ prerequisite 
skills have to be specified.

From this skill assignment, a competence structure and a 
lesson structure can be derived with the `CDSS` functions. These functions cover the whole workflow from reading the skill
assignments from spreadsheet files up to the determination of surmise functions or surmise relations.

The resulting structure may well need further, manual refinement, e.g. with respect to falsely equivalent skills (see [below](#sec:dealing-with-equivalence-classes)).

# Skill assignment data files
Information on the assignment of skills to learning objects can be specified in the three most standard spreadsheet
file formats (CSV, ODS, and XLSX). For ODS and XLSYX files, two sheets are expected specifying assignment of taught and required skills respectively. For the CSV case, two distinct files are expected because CSV does not provide for multi-sheet files.

Each sheet (or CSV file) contains a table with two columns, learning object id and skill id. For each skill assigned to a learning object, there is a separate row. The table may contain a header row; the reading functions (see below) assume by
default that it does.

# Workflow for the general case
Figure 1 shows the workflow for the general case, i.e. derivation of a surmise function from skill assignments in a course
where (some) competencies may be taught by several, alternative learning objects. The complete workflow is implemented in the
`cdss_wf_read_skill_assignment()` function but all individual steps can also be called individually.

![Figure 1: General CDSS Workflow](CDSS_Workflow.png){width=25%}

_Figure 1: General CDSS Workflow_

The first step is to read the skill assignment. There are three functions for that allowing for three different spreadsheet
file formats: `cdss_read_skill_assignment_csv()`, `cdss_read_skill_assignment_ods()`, and `cdss_read_skill_assignment_xlsx()`
for CSV (comma-separated values) files, LibreOffice/OpenOffice files, or Excel files, respectively. The tables read from the file are then converted into a skill assignment object after checking compliance to the properties of such objects.

The next conversions are from a skill assignment to a skill multi-assignment and a complete skill multi-assignment using
`cdss_sa2sma()` and `cdss_sma2csma()`. In the final step, a surmise function is determined with `cdss_csma2sf()`. 


# Workflow for the simplified case
In the _simplified case,_ there is exactly one teaching learning object for each skill (there may still be several taught
skills assigned to the same learning object). In this case, the resulting structures of skills and learning objects, respectively, are quasi-ordinal knowledge spaces.

Figure 2 shows the respective workflow which is briefly described underneath.

![Figure 2: Simplified CDSS Workflow](CDSS_Simplified_Workflow_Skills.png){width=70%} 

_Figure 2: Simplified CDSS Workflow_

After reading a skill assignment object using one of the functions mentioned in the [previous section](#sec:workflow-for-the-general-case), the applicability of this workflow is checked, i.e. whether there is truly
only one teaching learning object per skill. In the positive case, an _attribution relation_ can be derived using `cdss_sa2ar_skill()` which can then be closed to a surmise relation by callin `cdss_close_ar()`.

# Learning object structures
It is also possible to derive structures on the learning objects from the skill assignment. Currently, there exist three respective functions:

- `cdss_lo_csma2sf()` determines a surmise function on the set of learning objects based on a complete skill 
   multi-assignment object.
- `cdss_lo_sa2af()` determines an attribution function from a skill assignment object. However, currently, there is no 
   function available to close the attribution function to a surmise function.
- `cdss_lo_sa2ar()` derives an attribution relation from a skill assignment object. This attribution relation can then 
   be closed to a surmise relation using `cdss_close_ar()`. This function is only applicable in the aforementioned 
   _simple case_.

Surmise relations on learning objects may be applied for adaptive teaching. For example, [Moodle](https://moodle.org)
allows to specify prerequisite relations betgween diferent lessons/learning objects.

# Further processing

## General further processing with the `kstMatrix` package
The surmise functions and surmise relations built following the above workflows can be further processed using the
`kstMatrix` package (Hockemeyer, Steiner, & Wong, 2026). In a first step, the respective basis can be determined using 
`kstMatrix::kmbasis()` method. This opens the way to other tasks like constructing the knowledge space, plotting space 
or basis, or simulating response patterns from the space.

## Dealing with equivalence classes
One issue with the CDSS approach is that learning objects may teach multiple skills. This often leads to equivalence
classes of skills where the skills are not really equivalent but the equivalence is just induced by the connection to 
the same learning object. A manual fine-tuning may well be appropriate.

One could determine the equivalence classes using `kstmatrix::kmnotions()`. After saving the basis to a spreadsheet file 
with `kstIO::write_kbase()`, this file can be edited with the respective spreadsheet program. Simply add rows which
lead to a structure within the equivalence class(es). Afterward re-import the changed basis into R with `kstIO::read_kbase()` and process that changed basis further.

# Examples

## General workflow
In a first example, we do the whole workflow at once for a skill assignment table which does not describe a surmise relation, i.e. there are some skills taught by several learning objects. The resulting `sflist` contains `NULL` values for `srs` and `srl` accordingly. In a final step, we determine and plot the basis of the resulting skill space.
``` {r example}
fpath <- system.file("extdata", "SkillAssignment.xlsx", package="CDSS")
sflist <- cdss_wf_read_skill_assignment(fpath)
sflist
b <- kstMatrix::kmbasis(sflist$sfs)
b
```
Figure 3 shows the Hasse diagram of the basis (using the `plot()` method of the `kstMatrix` package).
```{r basis-figure, echo=FALSE}
pl <- plot(b)
# rsvg::rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), paste0(tempdir(), file="/plot3.png", width=2000))
bm <- rsvg::rsvg(charToRaw(DiagrammeRsvg::export_svg(pl)), width=1500)
png::writePNG(bm, paste0(tempdir(), file="/plot3.png"))
```
!["Example basis"](`{r} paste0(tempdir(), "/plot3.png")`){width=65%}

_Figure 3: Derived example basis_

_**Remarks**_

1. This basis does not belong to a quasi-ordinal space and, therefore, cannot be represented by a surmise relation:
   for example, the intersection of the states _{a,c}_ and _{b,c}_, i.e. _{c}_ is not a state.
2. The skills _f_ and _g_ are equivalent (cf. [previous section](#sec:dealing-with-equivalence-classes).
3. For technical reasons (with grViz and RMarkdown) the plot is manually included here but if you execute the code,
   e.g., in RStudio, the plot will be shown in the Viewer pane.
   
## Skill assignment tables
The skill assignment tables are given below, first the taught and then the required skills.
``` {r sat, echo=FALSE}
fpath <- system.file("extdata", "SkillAssignment-T.csv", package="CDSS")
read.csv(fpath)
fpath <- system.file("extdata", "SkillAssignment-R.csv", package="CDSS")
read.csv(fpath)
```

Please note that learning objects 1 and 2 do not have any prerequisite skills.

## Workflow for the simplified case
In this example, we do the individual steps. We use a derivation of the skill assignment used [above](#sec:general-workflow). First, we read the file and check whether it can be represented through a surmise relation.
```{r simplified}
fpath <- system.file("extdata", "SkillAssignment_SR.xlsx", package="CDSS")
sa <- cdss_read_skill_assignment_xlsx(fpath)
sa
cdss_sa_describes_sr(sa)
```
Subsequently, we build the attribution relation and its closure to a surmise relation.
```{r surmise}
ar <- cdss_sa2ar_skill(sa)
ar
sr <- cdss_close_ar(ar)
sr
```
Figure 4 shows the Hasse diagram of the surmise relation (again using the `plot()` method of the `kstMatrix` package).
```{r surmise-figure, echo=FALSE}
pl <- plot(sr)
# rsvg::rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), paste0(tempdir(), file="/plot4.png", width=1000))
bm <- rsvg::rsvg(charToRaw(DiagrammeRsvg::export_svg(pl)), width=1000)
png::writePNG(bm, paste0(tempdir(), file="/plot4.png"))
```
!["Example surmise relation"](`{r} paste0(tempdir(), "/plot4.png")`){width=19%}

_Figure 4: Derived example surmise system_

Please note that the skills _f_ and _g_ are equivalent.

# References
- Hockemeyer C, Steiner P, & Wong W (2026). _kstMatrix: Basic Functions in Knowledge Space Theory Using Matrix
Representation_. doi:10.32614/CRAN.package.kstMatrix, R package
version 2.3-1, <https://CRAN.R-project.org/package=kstMatrix>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_csma2sf.R
\name{cdss_csma2sf}
\alias{cdss_csma2sf}
\title{Derive a surmise function from a complete skill multi-assignment}
\usage{
cdss_csma2sf(csma)
}
\arguments{
\item{csma}{Skill multi-assignment to be completed}
}
\value{
Object of class \code{cdss_csma}.
}
\description{
\code{cdss_csma2sf} expects a complete skill multi-assignment object 
and returns the corresponding surmise function on the set of skills.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_circular_requirements.R
\name{cdss_circular_requirements}
\alias{cdss_circular_requirements}
\title{Vector of learning objects requiring and teaching the same skill}
\usage{
cdss_circular_requirements(sa)
}
\arguments{
\item{sa}{Skill assignment}
}
\value{
Vector of learning objects
}
\description{
\code{cdss_circular_requirements} expects skill assignment and returns
a vector of learning objects which require a skill that they teach.
}
\seealso{
Other Functions testing validity of skill assignments: 
\code{\link{cdss_missing_los}()},
\code{\link{cdss_nonteaching_los}()},
\code{\link{cdss_sa_compliance}()}
}
\concept{Functions testing validity of skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_read_skll_assignments_xlsx.R
\name{cdss_read_skill_assignment_xlsx}
\alias{cdss_read_skill_assignment_xlsx}
\title{Read an assignment of taught and required skills for a set of learning objects from an XLSX-file.}
\usage{
cdss_read_skill_assignment_xlsx(
  filename,
  taughtname = "Taught",
  requiredname = "Required",
  warnonly = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{filename}{Name of the XLSX-file}

\item{taughtname}{Name of the sheet with required assignment (default = "Taught")}

\item{requiredname}{Name of the sheet with required assignment (default = "Required")}

\item{warnonly}{Are non-compliant SAs allowed? (default = FALSE)}

\item{verbose}{Verbosity of compliance test (default = TRUE)}
}
\value{
List of two binary matrices, "taught" and "required".
}
\description{
\code{cdss_read_skill_assignment_xlsx} expects an XLSX-file with two sheets assigning taught and
required, respectively, skills to learning objects with two columns each. The first
column contains the IDs of learning objects and the second row the IDs of single skills
required or taught, respectively, by this learning object.
It returns a list of two binary matrices, "taught" and "required". Each matrix has one
row per learning object and one column per skill. The cells contain a "1" if the skill
is taught or required, respectively, by the learning object and a "0" otherwise,
}
\seealso{
Other functions reading skill assignments: 
\code{\link{cdss_read_skill_assignment_csv}()},
\code{\link{cdss_read_skill_assignment_ods}()},
\code{\link{cdss_wf_read_skill_assignment}()}
}
\concept{functions reading skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_lo_sa2ar.R
\name{cdss_lo_sa2ar}
\alias{cdss_lo_sa2ar}
\title{Create an attribution relation on learning objects from a skill assignment.}
\usage{
cdss_lo_sa2ar(sa)
}
\arguments{
\item{sa}{Skill assignment object}
}
\value{
attribution relation or NULL
}
\description{
\code{cdss_lo_sa2ar} expects a skill assignment and derives an attribution relation
 on learning objects if the skill assignment fulfills the necessary conditions, i.e. if  
 there is only one teaching LO per skill.
}
\seealso{
Other functions deriving skill structures from skill assignments: 
\code{\link{cdss_sa2ar_skill}()},
\code{\link{cdss_sa_describes_sr}()}
}
\concept{functions deriving skill structures from skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_read_skll_assignments_csv.R
\name{cdss_read_skill_assignment_csv}
\alias{cdss_read_skill_assignment_csv}
\title{Read an assignment of taught and required skills for a set of learning objects from CSV-files.}
\usage{
cdss_read_skill_assignment_csv(
  taught,
  required,
  header = TRUE,
  sep = ",",
  dec = ".",
  warnonly = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{taught}{CSV-file with assignments of taught competencies to learning objects}

\item{required}{CSV-file with assignments of required competencies to learning objects}

\item{header}{Boolean specifying whether the CSV-files contain a header line (default = TRUE)}

\item{sep}{Column separator (default ",")}

\item{dec}{Decimal point character (default ".")}

\item{warnonly}{Are non-compliant SAs allowed? (default = FALSE)}

\item{verbose}{Verbosity of compliance test (default = TRUE)}
}
\value{
List of two binary matrices, "taught" and "required".
}
\description{
\code{cdss_read_skill_assignment} expects two CSV-files with two columns each. The first
column contains the IDs of learning objects and the second row the IDs of single skills
required or taught, respectively, by this learning object.
It returns a list of two binary matrices, "taught" and "required". Each matrix has one
row per learning object and one column per skill. The cells contain a "1" if the skill
is taught or required, respectively, by the learning object and a "0" otherwise,
}
\seealso{
Other functions reading skill assignments: 
\code{\link{cdss_read_skill_assignment_ods}()},
\code{\link{cdss_read_skill_assignment_xlsx}()},
\code{\link{cdss_wf_read_skill_assignment}()}
}
\concept{functions reading skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_wf_read_skill_assignment.R
\name{cdss_wf_read_skill_assignment}
\alias{cdss_wf_read_skill_assignment}
\title{Read an assignment of taught and required skills for a set of learning objects from file and
do the whole workflow up to a surmise function on skills}
\usage{
cdss_wf_read_skill_assignment(
  filename,
  filename2 = NULL,
  filetype = "auto",
  taughtname = "Taught",
  requiredname = "Required",
  header = TRUE,
  sep = ",",
  dec = ".",
  warnonly = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{filename}{Name of the file (in case of CSV files the one with TAUGHT assignments)}

\item{filename2}{Name of the CSV file with REQUIRED assignments (if applicable)}

\item{filetype}{Type of the file, allowed values are "auto", "ODS", "XLSX", and "CSV"}

\item{taughtname}{Name of the sheet with required assignment (default = "Taught")}

\item{requiredname}{Name of the sheet with required assignment (default = "Required")}

\item{header}{Boolean specifying whether the CSV-files contain a header line (default = TRUE)}

\item{sep}{Column separator for CSV files (default ",")}

\item{dec}{Decimal point character for CSV files (default ".")}

\item{warnonly}{Are non-compliant SAs allowed? (default = FALSE)}

\item{verbose}{Verbosity of compliance test (default = TRUE)}
}
\value{
List of four elements: sfs (surmise function between skills),
                               sfl (surmise function between learning objects)
                               srs (surmise relation between skills, if available; NULL otherwise)
                               srl (surmise relation between learning objectrs, if available; NULL otherwise)
}
\description{
\code{cdss_wf_read_skill_assignment} expects an ODS or XLSX file with two sheets assigning 
taught and required, respectively, skills to learning objects with two columns each. 
Alternatively, two CSV files can be specified. In the sheets/CSV files, the first
column contains the IDs of learning objects and the second row the IDs of single skills
required or taught, respectively, by this learning object.
It returns a list of two binary matrices, "taught" and "required". Each matrix has one
row per learning object and one column per skill. The cells contain a "1" if the skill
is taught or required, respectively, by the learning object and a "0" otherwise,
}
\seealso{
Other functions reading skill assignments: 
\code{\link{cdss_read_skill_assignment_csv}()},
\code{\link{cdss_read_skill_assignment_ods}()},
\code{\link{cdss_read_skill_assignment_xlsx}()}
}
\concept{functions reading skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_sa2sma.R
\name{cdss_sa2sma}
\alias{cdss_sa2sma}
\title{Convert skill assignment matrices to skill multi-assignment}
\usage{
cdss_sa2sma(sa)
}
\arguments{
\item{sa}{Skill assignment object}
}
\value{
Object of class \code{cdss_sma}.
}
\description{
\code{cdss_sa2sma} expects a list of two matrices (\code{taught} and \code{required}) of a skill
assignment. It returns a skill multi-assignment object.
}
\seealso{
Other functions building skill (multi) assignment matrices: 
\code{\link{cdss_lo_csma2sf}()},
\code{\link{cdss_lo_sa2af}()},
\code{\link{cdss_tables2sa}()}
}
\concept{functions building skill (multi) assignment matrices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CDSS-package.R
\name{CDSS}
\alias{CDSS}
\alias{CDSS-package}
\title{CDSS: Course dependent skill structures}
\description{
The \code{CDSS} package provides functions for a complete workflow from
skill assignment tables to surmise mappings on the sets of skills and
learning objects, respectively.
}
\section{Suggested workflow for the general case}{

\enumerate{
\item Read the skill assignment using one of the \code{read_skill_assignments_xxx()} functions.
\item Check the compliance to the definition for skill assignments using \code{\link[=cdss_sa_compliance]{cdss_sa_compliance()}}.
\item Convert the skill assignment into a skill multi-assignment using \code{\link[=cdss_sa2sma]{cdss_sa2sma()}}.
\item Close the skill multi-assignment under completion using \code{\link[=cdss_sma2csma]{cdss_sma2csma()}}.
\item Compute the surmise function on skills using \code{\link[=cdss_csma2sf]{cdss_csma2sf()}}.
\item Continue with functions from the \code{kstMatrix} package, e.g., to obtain a basis and further on a skill space.
}
}

\section{Suggested workflow for the special case of one LO per skill}{

\enumerate{
\item Read the skill assignment using one of the \code{read_skill_assignments_xxx()} functions.
\item Check whether the skill assignment allows for the derivation of a surmise relation using \code{\link[=cdss_sa_describes_sr]{cdss_sa_describes_sr()}}.
\item If yes, derive an attribution relation from the skill assignment using \code{\link[=cdss_sa2ar_skill]{cdss_sa2ar_skill()}}.
\item Close the attribution relation to a surmise relation using \code{\link[=cdss_close_ar]{cdss_close_ar()}}.
\item Continue with functions from the \code{kstMatrix} package, e.g., to obtain a basis and further on a skill space.
}
}

\section{Data files}{

The installation of this package includes several data files as examples in the
\code{extdata} sub directory (see the Examples below for how to access the files there).
There are four data sets, \code{KST}, \code{KST-Intro}, \code{SkillAssignment}, and
\code{ErroneousSkillAssignment}. The \code{SkillAssignment} data set is available in
three formats, ODS, XLSX, and CSV (in CSV format, there are two files each,
\code{SkillAssignment-R.csv} and \code{SkillAssignment-T.csv}, for required and taught skills, respectively).
The other three data sets are available in ODS format only.

\code{SkillAssignment} and \code{ErroneousSkillAssignment} are small example data sets
where the latter fails for \code{cdss_sa_compliance()}. \code{KST} contains a skill
assignment for the course on knowledge space theory under \url{https://moodle.qhelp.eu/}.
\code{KST-Intro} contains the reduction of \code{KST} to the first chapter of
that course.
}

\section{References}{

Hockemeyer, C. (2022). Building Course-Dependent Skill Structures - Applying
Competence based Knowledge Space Theory to Itself. Manuscript in preparation.
}

\section{Acknowledgements}{

The creation of this R package was financially supported by the Erasmus+ Programme
of the European Commission through the QHELP project (\url{https://qhelp.eu/}).
}

\examples{
library(readODS)
fpath <- system.file("extdata", "SkillAssignment.ods", package="CDSS")
sa <- cdss_read_skill_assignment_ods(fpath)
sa
sma <- cdss_sa2sma(sa)
sma
csma <- cdss_sma2csma(sma)
csma
sf <- cdss_csma2sf(csma)
sf

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_sma2csma.R
\name{cdss_sma2csma}
\alias{cdss_sma2csma}
\title{Complete a skill multi-assignment}
\usage{
cdss_sma2csma(sma, allowcycles = FALSE)
}
\arguments{
\item{sma}{Skill multi-assignment to be completed}

\item{allowcycles}{Whether prerequisite cycles should be allowed (default = FALSE)}
}
\value{
Object of class \code{cdss_csma}.
}
\description{
\code{cdss_sma2csma} expects a skill multi-assignment object and returns
the corresponding complete skill multi-assignment. 
If this would involve cycles, the function stops by default - except if 
\code{allowcycles} is set to \code{TRUE}. In that case, the result may
be ill-defined!
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_reduce_sf.R
\name{cdss_reduce_sf}
\alias{cdss_reduce_sf}
\title{Reduce a surmise function with respect to item equivalence}
\usage{
cdss_reduce_sf(sf)
}
\arguments{
\item{sf}{Surmise function}
}
\value{
Surmise function reduced by equivalences
}
\description{
\code{cdss_reduce_sf} takes a surmise function and returns its
reduction to non-equivalent items.
}
\seealso{
Other Utility functions: 
\code{\link{cdss_binary_matrix_product}()},
\code{\link{cdss_close_ar}()}
}
\concept{Utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_binary_matrix_product.R
\name{cdss_binary_matrix_product}
\alias{cdss_binary_matrix_product}
\title{Compute a binary matrix product}
\usage{
cdss_binary_matrix_product(m, n)
}
\arguments{
\item{m}{Binary matrix}

\item{n}{Binary matrix}
}
\value{
Boolean matrix product of m and n
}
\description{
\code{cdss_binary_matrix_product} expects two binary matrices and computes there Boolean product.
}
\seealso{
Other Utility functions: 
\code{\link{cdss_close_ar}()},
\code{\link{cdss_reduce_sf}()}
}
\concept{Utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_close_ar.R
\name{cdss_close_ar}
\alias{cdss_close_ar}
\title{Close an attribution relation to get a surmise relation.}
\usage{
cdss_close_ar(ar)
}
\arguments{
\item{ar}{Attribution relation matrix}
}
\value{
surmise relation or NULL
}
\description{
\code{cdss_close_ar} expects a quadratic binary matrix and closes it under
reflexivity and transitivity.
}
\seealso{
Other Utility functions: 
\code{\link{cdss_binary_matrix_product}()},
\code{\link{cdss_reduce_sf}()}
}
\concept{Utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_missing_los.R
\name{cdss_missing_los}
\alias{cdss_missing_los}
\title{Vector of skills without teaching learning objects.}
\usage{
cdss_missing_los(sa)
}
\arguments{
\item{sa}{SKill assignment}
}
\value{
Vector of skills
}
\description{
\code{cdss_missing_los} expects a skill assignment and returns a vector
of skills which are not taught by any learning object.
}
\seealso{
Other Functions testing validity of skill assignments: 
\code{\link{cdss_circular_requirements}()},
\code{\link{cdss_nonteaching_los}()},
\code{\link{cdss_sa_compliance}()}
}
\concept{Functions testing validity of skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_lo_csma2sf.R
\name{cdss_lo_csma2sf}
\alias{cdss_lo_csma2sf}
\title{Derive a surmise function between learning objects from a complete skill multi-assignment}
\usage{
cdss_lo_csma2sf(csma)
}
\arguments{
\item{csma}{Complete skill multi-assignment object}
}
\value{
Object of class \code{cdss_sf} (attribution function).
}
\description{
\code{cdss_lo_csma2sf} expects a complete skill multi-assignment and derives a surmise
function on the set of learning objects.
}
\seealso{
Other functions building skill (multi) assignment matrices: 
\code{\link{cdss_lo_sa2af}()},
\code{\link{cdss_sa2sma}()},
\code{\link{cdss_tables2sa}()}
}
\concept{functions building skill (multi) assignment matrices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_sa_describes_sr.R
\name{cdss_sa_describes_sr}
\alias{cdss_sa_describes_sr}
\title{Check whether a surmise relation can be derived from a given skill assignment.}
\usage{
cdss_sa_describes_sr(sa, verbose = FALSE)
}
\arguments{
\item{sa}{Skill assignment object}

\item{verbose}{Flag, default is FALSE}
}
\value{
Logical value
}
\description{
\code{cdss_sa_describes_sr} expects a list of two matrices (\code{taught} and \code{required}) of a skill
assignment. It returns TRUE if the skill assignment describes a surmise relation (i.e. there is
only one teaching LO per skill) and FALSE.
}
\seealso{
Other functions deriving skill structures from skill assignments: 
\code{\link{cdss_lo_sa2ar}()},
\code{\link{cdss_sa2ar_skill}()}
}
\concept{functions deriving skill structures from skill assignments}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_lo_sa2af.R
\name{cdss_lo_sa2af}
\alias{cdss_lo_sa2af}
\title{Determine Attribution function for LOs from a skill assignment}
\usage{
cdss_lo_sa2af(sa)
}
\arguments{
\item{sa}{Skill assignment object}
}
\value{
Object of class \code{cdss_af} (attribution function).
}
\description{
\code{cdss_lo_sa2af} expects a skill assignment and derives an attribution
function on the set of learning objects.
}
\seealso{
Other functions building skill (multi) assignment matrices: 
\code{\link{cdss_lo_csma2sf}()},
\code{\link{cdss_sa2sma}()},
\code{\link{cdss_tables2sa}()}
}
\concept{functions building skill (multi) assignment matrices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_tables2sa.R
\name{cdss_tables2sa}
\alias{cdss_tables2sa}
\title{Build matrices of taught and required, respectively, skills for learning objects from
respective tables.}
\usage{
cdss_tables2sa(taught, required)
}
\arguments{
\item{taught}{Data table containing the assignment of taught skills to learning objects}

\item{required}{Data table containing the assignment of required skills to learning objects}
}
\value{
List of two binary matrices, "taught" and "required".
}
\description{
\code{cdss_tables2sa} expects two data frames with two columns each. The first
column contains the IDs of learning objects and the second row the IDs of single skills
required or taught, respectively, by this learning object.
It returns a list of two binary matrices, "taught" and "required". Each matrix has one
row per learning object and one column per skill. The cells contain a "1" if the skill
is taught or required, respectively, by the learning object and a "0" otherwise.
}
\seealso{
Other functions building skill (multi) assignment matrices: 
\code{\link{cdss_lo_csma2sf}()},
\code{\link{cdss_lo_sa2af}()},
\code{\link{cdss_sa2sma}()}
}
\concept{functions building skill (multi) assignment matrices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdss_sa_compliance.R
\name{cdss_sa_compliance}
\alias{cdss_sa_compliance}
\title{Check whether a skill assignment is compliant to the CDCS conditions.}
\usage{
cdss_sa_compliance(sa, warnings = FALSE)
}
\arguments{
\item{sa}{Skill assignment}

\item{warnings}{Toggles whether warnings should be printed}
}
\value{
Boolean
}
\description{
\code{cdss_sa_compliance} expects a skill assignment and checks whether
it is compliant to the conditions for CDCS.
}
\seealso{
Other Functions testing validity of skill assignments: 
\code{\link{cdss_circular_requirements}()},
\code{\link{cdss_missing_los}()},
\code{\link{cdss_nonteaching_los}()}
}
\concept{Functions testing validity of skill assignments}
